
/*
Open-source "snowflake" drawing app with true vector output.
Click the "save" button to create a PDF document of your snowflake,
suitable for editing in Inkscape, Corel, Illustrator, or so forth.

   
Written by Windell H. Oskay, www.evilmadscientist.com
Copyright 2008, all rights reserved.
Distributed under the GPL 3.0.

Saves files named "snowflake-####.pdf" in the directory where this .pde file
resides. 

 */


import processing.pdf.*;

void DrawVertex(float x,float y,float angle)
{    // Angle is in radians

  vertex(x*cos(angle) + y*sin(angle), y*cos(angle) - x*sin(angle));

}

float StartTime;
float timeNow;
float LastActiveTime;
float xValues[];
float yValues[];
int SideLength;
color circleColor;
color bgColor;
color flakeColor;
color segmentColor;
int CircleDia;
int mtCircleDia;
boolean dragging; 
int MovePoint;
boolean pointsActive;
boolean pointsActiveOld;
color emptyCircle;

boolean overCircle(int x, int y, int diameter) 
{
  float disX = x - mouseX;
  float disY = y - mouseY;
  if(sqrt(sq(disX) + sq(disY)) < diameter/2 ) {
    return true;
  } 
  else {
    return false;
  }
}


  boolean overRect(int x, int y, int width, int height) 
  {
    if (mouseX >= x && mouseX <= x+width && 
      mouseY >= y && mouseY <= y+height) {
      return true;
    } 
    else {
      return false;
    }
  }










void setup() 
{
 
  StartTime = millis();
  LastActiveTime = StartTime;
  size(640, 640);
  strokeWeight(.25);
  smooth();
  dragging = false; 
  pointsActive = false;

  bgColor = color(70, 70, 200);
  background(bgColor);
  flakeColor = 0;  
  segmentColor = 255;
  CircleDia =  10;
  circleColor = color(255, 128, 0);
  
  mtCircleDia =  10;
  emptyCircle = color(255,128,0,0);

  stroke(0);

  SideLength = width*9/20;

  xValues = new float[2];
  yValues = new float[2];

  xValues[0] = 0;
  xValues[1] = SideLength*sqrt(3)/4; 

  yValues[0] = -SideLength;
  yValues[1] = -0.75*SideLength; 

  
 textFont(loadFont("Ziggurat.vlw"), 20); 
 
  pushMatrix();
  translate(width/2,height/2);
  drawFlake();
  popMatrix();

}



void mousePressed() { 

  int i,len,xt,yt; 

  len = xValues.length;
  dragging = false;

  i = 0;
  while (i < len)
  { 
    if (overCircle((int) xValues[i] + width/2, (int) yValues[i] + height/2, CircleDia))
    {
      dragging = true; 
      MovePoint = i; 
      i = len; // break
    }

    i++;

  }



  if ( dragging == false)
  {
    i = 0;
    while (( i + 1) < len) 
    {

      xt = (int) (xValues[i] + xValues[i+1] )/2;
      yt = (int) (yValues[i] + yValues[i+1] )/2;    


      if (overCircle (xt + width/2, yt+ height/2,  CircleDia))
      {
        dragging = true; 
        MovePoint = i+1;

        xValues = splice(xValues, xt, i+1);
        yValues = splice(yValues, yt, i+1); 
        i = len; // break    
      }

      i++;
    }
  } 


if (overRect(50, 600, 100, 50) )  // Clear
{
  xValues = subset(xValues, 0, 2);
  yValues = subset(yValues, 0, 2); 
  
  xValues[0] = 0;
  xValues[1] = SideLength*sqrt(3)/4; 

  yValues[0] = -SideLength;
  yValues[1] = -0.75*SideLength; 
  
  pointsActive = true;
}



if (overRect(515, 600, 75, 25) )  // Save
{ 
  beginRecord(PDF, "snowflake-####.pdf"); 
   pushMatrix();
     translate(width/2,height/2);
 background(255);
stroke(0);
strokeWeight(1);


drawFlake();
popMatrix();

endRecord();

    strokeWeight(0);
    stroke(255);
    
}


}

void mouseReleased() {
  dragging = false;
}



void drawFlake()
{

  float angle;
  int i,j;
  fill(flakeColor);
  strokeWeight(.5); 
  beginShape(POLYGON);
  i = 0;
  while (i < 6)
  {

    angle = -i*PI/3;

    xValues = reverse(xValues);
    yValues = reverse(yValues);
    j = 0;
    while (j < xValues.length)
    { 
      DrawVertex(-1*xValues[j],yValues[j],angle);  
      j++;
    }

    xValues = reverse(xValues);
    yValues = reverse(yValues);

    j = 0;

    while (j < xValues.length)
    { 
      DrawVertex(xValues[j],yValues[j],angle);  
      j++;
    }  
    i++;
  }


  endShape(CLOSE);

}


void draw() 
{ 
  int i,j,len,xt,yt;
  float xf,yf;

  len = xValues.length;
  
  timeNow = millis(); 
  
  
  if (pointsActive) 
     LastActiveTime = timeNow;
  
if ((timeNow - LastActiveTime) < 1000)
   pointsActiveOld = true;
else
   pointsActiveOld = false;
   
pointsActive = false;
 


  pushMatrix();

  translate(width/2,height/2);

  background(bgColor);
  drawFlake();
  popMatrix();



  if ((timeNow - StartTime) < 12750)
  {

    fill(segmentColor);
    strokeWeight(0);
    stroke(255);
    pushMatrix();
    translate(width/2,height/2);
    beginShape(POLYGON);
    DrawVertex(0,0,0);  

    i = 0;
    while (i < xValues.length)
    { 
      DrawVertex(xValues[i],yValues[i],0);  
      i++;
    }
    DrawVertex(0,0,0);  
    endShape(CLOSE);
    popMatrix();


    j =  (int) (timeNow - StartTime) / 50;
    flakeColor = color(255,255,255,j);


    i = 0;
    while (i < len)
    {  
      {
        pushMatrix();
        translate(width/2,height/2);

        fill(red(circleColor),green(circleColor),blue(circleColor),255-j);
        //   fill(circleColor);
        ellipse(xValues[i], yValues[i], CircleDia, CircleDia);
        popMatrix();
      }


      if (( i + 1) < len)
      {

        xt = (int) (xValues[i] + xValues[i+1] )/2;
        yt = (int) (yValues[i] + yValues[i+1] )/2;    

        {
          pushMatrix();
          translate(width/2,height/2); 
          ellipse(xt, yt,CircleDia, CircleDia);
          popMatrix();
        }

      }

      i++;
    }

  }
  else
  {
    flakeColor = 255;
  }





  if ( dragging)
  {

pointsActive = true;

    if (MovePoint == 0)
    {
      xt = width/2;

      if (mouseY < height/2)
      {      
        yValues[MovePoint] = mouseY - height/2; 
        yt = mouseY;    
      }
      else
      {     
        yValues[MovePoint] = 0; 
        yt = height/2;    
      }



    }
    else if (MovePoint == len -1)    // Last point in array.
    {

      xf = mouseX - width/2;
      yf = mouseY - height/2; 

      xf = -yf/sqrt(3); 

      if (yf > 0)
      {
        xf = 0;
        yf = 0; 
      }

      xValues[MovePoint] = (int) xf;
      yValues[MovePoint] = (int) yf;

      xt = (int) ( width/2  + xf);
      yt = (int) ( height/2  + yf);   

    }
    else
    {
      xValues[MovePoint] = mouseX - width/2;
      yValues[MovePoint] = mouseY - height/2;   
      xt = mouseX;
      yt = mouseY;

    }
    fill(circleColor);

    ellipse(xt, yt,CircleDia, CircleDia); 

  }
  else
  {
    i = 0;
    while (i < len)
    { 
      if (overCircle((int) xValues[i] + width/2, (int) yValues[i] + height/2, CircleDia))
      {
         pointsActive=true; 
        pushMatrix();
        translate(width/2,height/2);

        fill(circleColor);
        ellipse(xValues[i], yValues[i], CircleDia, CircleDia);
        popMatrix();
      }
      else if (pointsActiveOld) 
      {   
        pushMatrix();
        translate(width/2,height/2);
        fill(emptyCircle);
        ellipse(xValues[i], yValues[i], mtCircleDia, mtCircleDia);
        popMatrix();

      }

      if (( i + 1) < len)
      {

        xt = (int) (xValues[i] + xValues[i+1] )/2;
        yt = (int) (yValues[i] + yValues[i+1] )/2;    

        if (overCircle (xt + width/2,yt+ height/2,  CircleDia)) 
        {
          pointsActive=true; 
          pushMatrix();
          translate(width/2,height/2);
          fill(circleColor);
          ellipse(xt, yt,CircleDia, CircleDia);
          popMatrix();
        } 
        else if (pointsActiveOld) 
        { 
          pushMatrix();
          translate(width/2,height/2);
          fill(emptyCircle);
          ellipse(xt, yt,mtCircleDia, mtCircleDia);
          popMatrix(); 
        }

 

      }

      i++;
    }

  }



// Draw text:

 fill(255,255,255,128);
  text("Clear", 50, 620);
 text("Save", 520, 620);


if (overRect(50, 600, 100, 50) )
{
//fill(circleColor);
//rect(45, 600, 75, 25);
fill(255);
text("Clear", 50, 620);
}


if (overRect(515, 600, 75, 25) )
{
//fill(circleColor);
//rect(515, 600, 75, 25);
fill(255); 
 text("Save", 520, 620);
}

 
} 

